/*
   sa1100.c

   Copyright  1999  University of Washington
   Portions Copyright  1998 Intel Corp.


   A collection of C/C++ utility functions for working with
   the SA-1100 microprocessor using the ARM SDT.

   Revision History:

      Created Sept 30, 1999 - Jeff Nichols
*/

#include "sa1100.h"


/***********************************************
  Interrupt Setup Functions
***********************************************/

void SetIrqLevel( unsigned uIrq, unsigned isFIQ ) {
  struct scm_ic_regs * icreg = (struct scm_ic_regs *)IC_BASE;

  if (isFIQ) {
    icreg->iclr |= uIrq;              // make irq a FIQ
  } else {
    icreg->iclr &= ~uIrq;             // make irq an IRQ
  }
}


void EnableIrq( unsigned uMask ) {
  struct scm_ic_regs * icreg = (struct scm_ic_regs *)IC_BASE;

  icreg->icmr |= uMask;               // unmask irqs
}


void DisableIRQ( unsigned uMask ) {
  struct scm_ic_regs * icreg = (struct scm_ic_regs *)IC_BASE;

  icreg->icmr &= ~uMask;              // disable interrupts for now
}


void DisableAllInts() {
  struct scm_ic_regs * icreg = (struct scm_ic_regs *)IC_BASE;

  icreg->icmr = 0;                    // disable interrupts for now
}


// Updates contents of 'vector' to contain branch instruction
// to reach 'routine' from 'vector'. Function return value is
// original contents of 'vector'.
//
// NB: 'Routine' must be within range of 32Mbytes from 'vector'.

unsigned Install_Handler( unsigned routine, unsigned *vector ) {
  unsigned vec, oldvec;

  vec = ((routine - (unsigned)vector - 0x8)>>2);
  if (vec & 0xff000000) { 
    return 0;
  }
  vec = 0xea000000 | vec;
  oldvec = *vector;
  *vector = vec;

  CleanAllCaches();

  return oldvec;
}


/***********************************************
  Busy Wait Functions
***********************************************/

void Wait( unsigned uSec ) {
  // waits the specified number of microseconds

  volatile unsigned *lTimerOSCRAddress= (unsigned *)OSCR;
  unsigned lValue, Time;

  Time = *lTimerOSCRAddress;

  lValue = Time + (uSec * TIMERTICK);
  if (lValue < Time) {
    while (Time < *lTimerOSCRAddress);
  }
  while (*lTimerOSCRAddress <= lValue);
}


/***********************************************
  Real Time Clock Functions
***********************************************/

void startRTC( void ) {
  struct scm_rtc_Regs * RTCRegs = (struct scm_rtc_Regs*)RTCBASE;
  RTCRegs->rttr = 0x8CCC;
}

void setRTCdivider( unsigned divider ) {
  struct scm_rtc_Regs * RTCRegs = (struct scm_rtc_Regs*)RTCBASE;

  RTCRegs->rttr = divider;
}

unsigned getRTC( void ) {
  struct scm_rtc_Regs * RTCRegs = (struct scm_rtc_Regs*)RTCBASE;

  return RTCRegs->rcnr;
}

void setRTC( unsigned val ) {
  // refer to the manual for some caveats about setting the real time clock

  struct scm_rtc_Regs * RTCRegs = (struct scm_rtc_Regs*)RTCBASE;

  RTCRegs->rcnr = val;
}


/***********************************************
  Timer Interrupt Functions
***********************************************/

void TimerSetup( unsigned timerSel, unsigned cnt ) {
  struct os_timer_regs *OSTimerRegs = (struct os_timer_regs*)OSMR_0;

  DisableTimerIRQ( IRQ_OSTIMER0+timerSel );

  OSTimerRegs->oscr = 0;                      // initialize timer counter register
  OSTimerRegs->osmr[timerSel] = cnt;          // set timer max count
  OSTimerRegs->oier |= 1 << timerSel;         // unmask timer irq
  OSTimerRegs->ossr  = 1 << timerSel;         // reset interrupt source

  EnableTimerIRQ( IRQ_OSTIMER0+timerSel );
}

void initTimer( unsigned timerSel, unsigned us ) {
  TimerSetup( timerSel, 0x24*us );
}

void EnableTimerIRQ( unsigned timerSel ) {
  SetIrqLevel( timerSel, FALSE );
  EnableIrq( timerSel );
}

void DisableTimerIRQ( unsigned timerSel ) {
  DisableIRQ( timerSel );
}
